/*
 * Copyright (C) 2005 Cesar Kallas
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * E-mail: cesarkallas@gmx.net
 * http://cesarkallas.soulivre.org
 *
 */

package com.estella.cliente;
import  com.estella.util.Log;
import  com.estella.util.Constantes;

import  java.net.Socket;
import  java.net.ServerSocket;
import  java.io.IOException;
import  java.io.DataInputStream;
import  java.io.DataOutputStream;

/**
* Funcoes para gerenciar conexoes de saida
* @author Cesar Kallas - cesarkallas@gmx.net
* @since 22/11/2005
*/
public class Conexao
{
  private Log LOG;
  private Constantes CON;
  private Socket socket;
	private DataInputStream receive;
	private DataOutputStream send;
  private String host;
  private int porta;

  /**
  * Instancia uma nova conexao
  * @param host String com o host no formato host.com ou 127.0.0.1
  * @param porta Inteiro contendo um numero de porta
  */
  public Conexao(String host, int porta)
  {
    CON = new Constantes();
    LOG = new Log();
    this.host = host;
    this.porta = porta;
  }
  public Conexao(Socket socket)
  {
    this.socket = socket;
  }

  /**
  * Cria um socket cliente, setando os parametros necessarios
  * @return true ou false / se a conexao foi criada e se modo leitura e gravacao estao prontos
  */  
  public boolean inicia()
	{
    if(socket == null)
      try {
        socket = new Socket(host, porta);
        socket.setSoTimeout(CON.SOCKET_TIMEOUT);
        socket.setTcpNoDelay(CON.SOCKET_TCPNODELAY);
        LOG.info("Conexao iniciada: " + socket);
      }
      catch(IOException FalhaConectar) {
        LOG.falha(FalhaConectar.getMessage() + " @ " + host + ":" + porta);
        return false;
      }
      catch(SecurityException Firewall) {
        LOG.atencao(Firewall.getMessage()  + " @ " + host + ":" + porta);
        return true;
      }
      catch(Exception FalhaGeral) {
        LOG.falha(FalhaGeral.getMessage()  + " @ " + host + ":" + porta);
        return false;
      }
    
    try {
      receive = new DataInputStream(socket.getInputStream());
      send = new DataOutputStream(socket.getOutputStream());
      LOG.info("Iniciando modo de leitura e gravacao no socket: " + host + ":" + porta);
      return true;
		}
    catch(IOException FalhaCanal) {
      LOG.falha(FalhaCanal.getMessage());
      return false;
    }
  }
  
  /**
  * Escreve dados no socket
  * @param socket - Criado na funcao (iniciar)
  * @param msg - String com a mensagem a ser escrita no socket
  * @return true ou false
  */
  public boolean send(String msg)
  {
    try {
      send.write(msg.getBytes());
      send.flush();
      LOG.info("Escrevendo no socket: " + socket);
      return true;
    }
    catch(Exception EnviarMsg) {
      LOG.falha(EnviarMsg.getMessage() + " @ " + host + ":" + porta);
      return false;
    }
  }
  
  /**
  * Recebe dados do socket
  * @param socket - Criado na funcao iniciar()
  * @return msg - String com a mensagem recebida ou null - Problema na recepcao de dados
  */
  public String read()
  {
    try {
      byte[] dados = new byte[receive.available()];
      receive.readFully(dados);
      BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(receive));
      LOG.info("Lendo o socket: " + socket);
      return (new String(dados));
    }
    catch(Exception ReceberMsg) {
      LOG.falha(ReceberMsg.getMessage() + " @ " + host + ":" + porta);
      return null;
    }
  }
}
